// PROJECT  :WDT328P
// PURPOSE  :Introduces the Watchdog Timer Interrupt as Scheduler to toggle Onboard LED
// COURSE   :ICS4U
// AUTHOR   :C. D'Arcy
// DATE     :2022 ?? ??
// MCU      :328P
// STATUS   :Working
// REFERENCE:http://darcy.rsgc.on.ca/ACES/Datasheets/ATMEGA328P.pdf
//          :**see C code on Page 53 
#include <Mega328P.h>       //consider placing OnBoard LED and prescaler assets in your library
volatile boolean timeout = false;

uint8_t state;
void setup() {
  // This doesn't work as there's too much hidden in the Arduino toolchain before this,
  // but this is the idea...
  //capture the state of the MCU Status Register
  state = MCUSR;  
  //clear all the available flags by writing a 1 to them...      
  MCUSR = 1 << WDRF | 1 << BORF | 1 << EXTRF | 1 << PORF;
  // display the source...
  Serial.begin(9600);
  while (!Serial);
  Serial.println("A System Reset has occurred"); //used to verify a System Reset
  Serial.println("MCUSR\t:" + String(state, BIN));
  Serial.println("PORF\t:" + String(state & (1 << PORF) ? "yes" : "no"));
  Serial.println("EXTRF\t:" + String(state & (1 << EXTRF) ? "yes" : "no"));
  Serial.println("BORF\t:" + String(state & (1 << BORF) ? "yes" : "no"));
  Serial.println("WDRF\t:" + String(state & (1 << WDRF) ? "yes" : "no"));
  Serial.end();

  // prepare for use of the Onboard LED

  // (temporarily) disable global interrupts through register use
                //aka cli();

  // clear the WDRF in the MCUSR  (don't want a Watchdog Reset)
  

  // call the setupWDT() function to configure the WDT to interrupt every 1s
  
  //reenable global interrupts
                //aka sei();  (enable global interrupts)
}

void setupWDT(uint8_t ps) {
  // Start timed sequence by setting the Change Enable and Enable bits simulatenously


  // We're allowed four clock cycles to set the new values from here -
  // Configure the WDTCSR for three behaviours...
  //  a) Interrupt Enable (WDIE)
  //  b) ONLY WD Interrupt and NOT WD System Reset  (WDE off)
  //  c) Set the prescaler (time-out) value using the parameter, ps (128K cycles~1s)

  // Finished setting new values, used 2 cycles -
}

// toggle the Onboard LED the fast way...
ISR(WDT_vect) {
  timeout = true;
}

//
void loop() {
  if (timeout) {
    timeout = false;
                 // toggle the Onboard LED the fast way...
  }
}
